/*
 * fmci-client.c
 *
 *  Copyright (C) 2010 Roman Rakus <rrakus@redhat.com>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#include <stdlib.h>

#include <dbus/dbus-glib-bindings.h>
#include <glib.h>

#include "fmci-client-bindings.h"
#include "fmci.h"

int
main(int argc, char** argv)
{
  GError* error = NULL;
  DBusGConnection *connection = NULL;
  DBusGProxy *proxy = NULL;

  char* version = NULL; // We use version as an example

  g_type_init();

  // Connect to DBus
  connection = dbus_g_bus_get(DBUS_BUS_SYSTEM, &error);

  if (!connection)
    {
      g_error("Failed to open connection to bus: %s", error->message);
      exit(1);
    }

  // Create proxy for our fmci interface
  proxy = dbus_g_proxy_new_for_name_owner(connection, FMCI_BUS_NAME,
      FMCI_OBJECT_PATH, FMCI_INTERFACE_NAME, &error);

  if (!proxy)
    {
      g_error("Failed to create proxy for name owner: %s", error->message);
      g_error_free(error);
      exit(1);
    }

  // Now we can call any of remote methods
  if (!org_fedoraproject_fmci_version(proxy, &version, &error))
    {
      g_error("Remote call for version failed: %s", error->message);
      g_error_free(error);
      exit(1);
    }

  g_print("We got version: %s\n", version);
  g_free(version);

  return 0;
}
